# -*- coding: utf-8 -*-

import os
import glob
import datetime
import urllib
from xml.etree.ElementTree import ElementTree
import re

def crawl(xml_path):
    try:
        urllib.urlretrieve('http://www.nicovideo.jp/ranking/fav/hourly/all?rss=2.0&lang=ja-jp', xml_path)
    except:
        return False

    return True

class Movie(object):
    def __init__(self, i, item):
        self.rank = i + 1
        link =  item.find('link').text
        m = re.search('/((sm|nm|so)[0-9]*?)$', link)
        self.link = m.group(1)
        description =  item.find('description').text.encode('utf-8')
        m = re.search('alt="(.*?)"', description)
        self.title = m.group(1)
        m = re.search('src="(.*?)"', description)
        self.thumbnail = m.group(1)
        m = re.search('<strong class="nico-info-number">([0-9,]*?)</strong>', description)
        self.pts = m.group(1)
        m = re.search('<strong class="nico-info-hourly-view">([0-9,]*?)</strong>', description)
        self.view = m.group(1)
        m = re.search('<strong class="nico-info-hourly-res">([0-9,]*?)</strong>', description)
        self.res = m.group(1)
        m = re.search('<strong class="nico-info-hourly-mylist">([0-9,]*?)</strong>', description)
        self.mylist = m.group(1)

    def get_thumbnail(self):
        if os.path.isfile('/ompy/nico/Original Images/' + self.link + '.jpg') or os.path.isfile('/ompy/nico/' + self.link + '.jpg'):
            return True

        try:
            urllib.urlretrieve(self.thumbnail, '/ompy/nico/' + self.link + '.jpg')
        except:
            return False

        return True

    def draw(self, ctx, position):
        left, top = position.x, position.y + (self.rank - 1) * 130 + 70

        ctx.set_source_rgb(1.0, 1.0, 1.0)

        #rank
        rank = str(self.rank)
        ctx.set_font_size(90)
        extents = ctx.text_extents(rank)
        x = left + 40 - extents[2] * 0.5
        y = top + 40 + extents[3] * 0.5
        ctx.move_to(x, y)
        ctx.show_text(rank)

        #pts
        pts = self.pts + 'pts'
        ctx.set_font_size(18)
        extents = ctx.text_extents(pts)
        x = left + 40 - extents[2] * 0.5
        y = top + 90 + extents[3] * 0.5
        ctx.move_to(x, y)
        ctx.show_text(pts)

        #title
        title = self.title
        ctx.set_font_size(25)
        extents = ctx.text_extents(title)
        x = left + 240
        y = top + 30 + extents[3] * 0.5
        ctx.move_to(x, y)
        ctx.show_text(title)

        #view comment mylist
        view = '再生:' + self.view
        ctx.set_font_size(20)
        extents = ctx.text_extents(view)
        x = left + 240
        y = top + 80 + extents[3] * 0.5
        ctx.move_to(x, y)
        ctx.show_text(view)

        res = 'コメント:' + self.res
        ctx.set_font_size(20)
        x += extents[2] + 20
        extents = ctx.text_extents(res)
        ctx.move_to(x, y)
        ctx.show_text(res)

        mylist = 'マイリスト:' + self.mylist
        ctx.set_font_size(20)
        x += extents[2] + 20
        extents = ctx.text_extents(mylist)
        ctx.move_to(x, y)
        ctx.show_text(mylist)

        #thumbnail
        try:
            surface = cairo.ImageSurface.create_from_png('/ompy/nico/PNG Images/' + self.link + '.png')
            x = left + 90
            y = top
            ctx.save()
            ctx.translate(x, y)
            ctx.set_source_surface(surface)
            ctx.paint()
            ctx.restore()
        except:
            pass

def draw_title(ctx, date, position):
    ctx.set_source_rgb(1.0, 1.0, 1.0)
    title = str(date.year) + '年' + str(date.month) + '月' + str(date.day) + '日' + str(date.hour - 1) + ':00-' + str(date.hour) + ':00' + 'のランキング' 
    ctx.set_font_size(50)
    extents = ctx.text_extents(title)
    x = position.x
    y = position.y + extents[3]
    ctx.move_to(x, y)
    ctx.show_text(title)

def ompy_params(d):
    d.floatDescs[0].name = 'Offset'
    d.pointDescs[0].name = 'Position'

def ompy_main(c):
    ctx = c.layer_out_context
    ctx.set_font_face(cairo.ToyFontFace('Osaka-Mono'))

    today = datetime.datetime.today()

    time = str(today.year) + str(today.month) + str(today.day) + str(today.hour)

    xml_path = '/ompy/nico/' + time + '.xml'

    if os.path.isfile(xml_path) == False:
        if crawl(xml_path) == False:
            print 'Error'
            return

    xmls = glob.glob('/ompy/nico/*.xml')

    if len(xmls) == 0:
        return

    offset = (len(xmls) - 1 + int(c.floats[0])) % len(xmls)

    xml_path = xmls[offset]

    tree = ElementTree(file=xml_path)
    root = tree.getroot()

    channel = root.find('channel')
    pub_date = channel.find('pubDate').text
    date = datetime.datetime.strptime(pub_date, '%a, %d %b %Y %H:%M:%S +0900')

    items = root.findall('.//item')

    movies = []

    within = 25

    for i in range(within):
        item = items[i]
        movies.append(Movie(i, item))

    for movie in movies:
        movie.get_thumbnail()

    position = c.points[0]

    draw_title(ctx, date, position)

    for movie in movies:
        movie.draw(ctx, position)
